@@ -25,6 +25,7 @@
 #include <string>
 #include <iterator>
 #include <QtCore/q20memory.h>
+#include <string_view>
 
 #include <stdarg.h>
 
@@ -331,21 +332,71 @@ public:
 #if QT_CONFIG(regularexpression)
     [[nodiscard]] QString section(const QRegularExpression &re, qsizetype start, qsizetype end = -1, SectionFlags flags = SectionDefault) const;
 #endif
-    [[nodiscard]] QString left(qsizetype n) const;
-    [[nodiscard]] QString right(qsizetype n) const;
-    [[nodiscard]] QString mid(qsizetype position, qsizetype n = -1) const;
 
-    [[nodiscard]] QString first(qsizetype n) const
+#if QT_CORE_REMOVED_SINCE(6, 7)
+    QString left(qsizetype n) const;
+    QString right(qsizetype n) const;
+    QString mid(qsizetype position, qsizetype n = -1) const;
+
+    QString first(qsizetype n) const;
+    QString last(qsizetype n) const;
+    QString sliced(qsizetype pos) const;
+    QString sliced(qsizetype pos, qsizetype n) const;
+    QString chopped(qsizetype n) const;
+#else
+    [[nodiscard]] QString left(qsizetype n) const &
+    {
+        if (size_t(n) >= size_t(size()))
+            return *this;
+        return first(n);
+    }
+    [[nodiscard]] QString left(qsizetype n) &&
+    {
+        if (size_t(n) >= size_t(size()))
+            return std::move(*this);
+        return std::move(*this).first(n);
+    }
+    [[nodiscard]] QString right(qsizetype n) const &
+    {
+        if (size_t(n) >= size_t(size()))
+            return *this;
+        return last(n);
+    }
+    [[nodiscard]] QString right(qsizetype n) &&
+    {
+        if (size_t(n) >= size_t(size()))
+            return std::move(*this);
+        return std::move(*this).last(n);
+    }
+    [[nodiscard]] QString mid(qsizetype position, qsizetype n = -1) const &;
+    [[nodiscard]] QString mid(qsizetype position, qsizetype n = -1) &&;
+
+    [[nodiscard]] QString first(qsizetype n) const &
     { verify(0, n); return sliced(0, n); }
-    [[nodiscard]] QString last(qsizetype n) const
+    [[nodiscard]] QString last(qsizetype n) const &
     { verify(0, n); return sliced(size() - n, n); }
-    [[nodiscard]] QString sliced(qsizetype pos) const
-    { verify(pos, 0); return QString(data() + pos, size() - pos); }
-    [[nodiscard]] QString sliced(qsizetype pos, qsizetype n) const
-    { verify(pos, n); return QString(data() + pos, n); }
-    [[nodiscard]] QString chopped(qsizetype n) const
+    [[nodiscard]] QString sliced(qsizetype pos) const &
+    { verify(pos, 0); return sliced(pos, size() - pos); }
+    [[nodiscard]] QString sliced(qsizetype pos, qsizetype n) const &
+    { verify(pos, n); return QString(begin() + pos, n); }
+    [[nodiscard]] QString chopped(qsizetype n) const &
     { verify(0, n); return sliced(0, size() - n); }
 
+    [[nodiscard]] QString first(qsizetype n) &&
+    {
+        verify(0, n);
+        resize(n);      // may detach and allocate memory
+        return std::move(*this);
+    }
+    [[nodiscard]] QString last(qsizetype n) &&
+    { verify(0, n); return sliced_helper(*this, size() - n, n); }
+    [[nodiscard]] QString sliced(qsizetype pos) &&
+    { verify(pos, 0); return sliced_helper(*this, pos, size() - pos); }
+    [[nodiscard]] QString sliced(qsizetype pos, qsizetype n) &&
+    { verify(pos, n); return sliced_helper(*this, pos, n); }
+    [[nodiscard]] QString chopped(qsizetype n) &&
+    { verify(0, n); return std::move(*this).first(size() - n); }
+#endif
     bool startsWith(const QString &s, Qt::CaseSensitivity cs = Qt::CaseSensitive) const;
     [[nodiscard]] bool startsWith(QStringView s, Qt::CaseSensitivity cs = Qt::CaseSensitive) const noexcept
     { return QtPrivate::startsWith(*this, s, cs); }
@@ -453,11 +504,13 @@ public:
             return *this;
         } else if constexpr (QtPrivate::IsCompatibleChar8Type<V>::value) {
             assign_helper_char8(first, last);
-            d.data()[d.size] = u'\0';
+            if (d.constAllocatedCapacity())
+                d.data()[d.size] = u'\0';
             return *this;
         } else {
             d.assign(first, last, [](QChar ch) -> char16_t { return ch.unicode(); });
-            d.data()[d.size] = u'\0';
+            if (d.constAllocatedCapacity())
+                d.data()[d.size] = u'\0';
             return *this;
         }
     }
@@ -914,8 +967,6 @@ public:
 
     inline bool isNull() const { return d->isNull(); }
 
-
-    bool isSimpleText() const;
     bool isRightToLeft() const;
     [[nodiscard]] bool isValidUtf16() const noexcept
     { return QStringView(*this).isValidUtf16(); }
@@ -951,6 +1002,7 @@ private:
                               Qt::CaseSensitivity cs = Qt::CaseSensitive) noexcept;
     static int localeAwareCompare_helper(const QChar *data1, qsizetype length1,
                                          const QChar *data2, qsizetype length2);
+    static QString sliced_helper(QString &str, qsizetype pos, qsizetype n);
     static QString toLower_helper(const QString &str);
     static QString toLower_helper(QString &str);
     static QString toUpper_helper(const QString &str);
@@ -1104,7 +1156,7 @@ QString QAnyStringView::toString() const
 // QString inline members
 //
 QString::QString(QLatin1StringView latin1)
-{ *this = QString::fromLatin1(latin1.data(), latin1.size()); }
+    : QString{QString::fromLatin1(latin1.data(), latin1.size())} {}
 const QChar QString::at(qsizetype i) const
 { verify(i, 1); return QChar(d.data()[i]); }
 const QChar QString::operator[](qsizetype i) const
