@@ -13,6 +13,7 @@
 #include <QTest>
 #include <QtTest/qsignalspy.h>
 
+#include <QtCore/private/qglobal_p.h> // for access to Qt's feature system
 #include <QtCore/qbuffer.h>
 #include <QtCore/qjsonobject.h>
 #include <QtCore/qjsondocument.h>
@@ -721,11 +722,26 @@ void tst_QRestAccessManager::text()
     // should consider the indicated charset and convert it to an UTF-16 QString => the returned
     // QString from text() should match with the original (UTF-16) QString.
 
-    // Successful UTF-8
+    // Successful UTF-8 (explicit)
     serverSideResponse.headers.insert("Content-Type:"_ba, "text/plain; charset=UTF-8"_ba);
     serverSideResponse.body = encUTF8(sourceString);
     VERIFY_TEXT_REPLY_OK;
 
+    // Successful UTF-8 (obfuscated)
+    serverSideResponse.headers["Content-Type:"_ba] = "text/plain; charset=\"UT\\F-8\""_ba;
+    serverSideResponse.body = encUTF8(sourceString);
+    VERIFY_TEXT_REPLY_OK;
+
+    // Successful UTF-8 (empty charset)
+    serverSideResponse.headers["Content-Type:"_ba] = "text/plain; charset=\"\""_ba;
+    serverSideResponse.body = encUTF8(sourceString);
+    VERIFY_TEXT_REPLY_OK;
+
+    // Successful UTF-8 (implicit)
+    serverSideResponse.headers["Content-Type:"_ba] = "text/plain"_ba;
+    serverSideResponse.body = encUTF8(sourceString);
+    VERIFY_TEXT_REPLY_OK;
+
     // Successful UTF-16
     serverSideResponse.headers.insert("Content-Type:"_ba, "text/plain; charset=UTF-16"_ba);
     serverSideResponse.body = encUTF16(sourceString);
@@ -741,9 +757,15 @@ void tst_QRestAccessManager::text()
     serverSideResponse.body = encUTF32(sourceString);
     VERIFY_TEXT_REPLY_OK;
 
-    // Successful UTF-32 with spec-wise allowed extra content in the Content-Type header value
+    // Successful UTF-32 with spec-wise allowed extra trailing content in the Content-Type header value
     serverSideResponse.headers.insert("Content-Type:"_ba,
-                                      "text/plain; charset = \"UTF-32\";extraparameter=bar"_ba);
+                                      "text(this is a \\)comment)/ (this (too)) plain; charset = \"UTF-32\";extraparameter=bar"_ba);
+    serverSideResponse.body = encUTF32(sourceString);
+    VERIFY_TEXT_REPLY_OK;
+
+    // Successful UTF-32 with spec-wise allowed extra leading content in the Content-Type header value
+    serverSideResponse.headers.insert("Content-Type:"_ba,
+                                      "text/plain; extraparameter=bar;charset    =     \"UT\\F-32\""_ba);
     serverSideResponse.body = encUTF32(sourceString);
     VERIFY_TEXT_REPLY_OK;
 
